import { concatAll as concatAllObs, distinctUntilChanged as distinctUntilChangedObs, endWith as endWithObs, exhaust as exhaustObs, ignoreElements as ignoreElementsObs, keep as keepObs, map as mapObs, mergeAll as mergeAllObs, observe as observeObs, onComplete as onCompleteObs, onError as onErrorObs, onNext as onNextObs, pipe as pipeObs, repeat as repeatObs, retry as retryObs, scan as scanObs, share as shareObs, startWith as startWithObs, subscribeOn as subscribeOnObs, switchAll as switchAllObs, take as takeObs, takeLast as takeLastObs, takeWhile as takeWhileObs, throttle as throttleObs, throttleFirst as throttleFirstObs, throttleFirstTime as throttleFirstTimeObs, throttleLast as throttleLastObs, throttleLastTime as throttleLastTimeObs, throttleTime as throttleTimeObs, timeout as timeoutObs, withLatestFrom as withLatestFromObs, } from "@reactive-js/observable";
class LiftedIteratorImpl {
    constructor(dispatcher, observable) {
        this.dispatcher = dispatcher;
        this.observable = observable;
    }
    dispatch(req) {
        this.dispatcher(req);
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
const liftImpl = (operator, dispatchOperator) => iterator => {
    const observable = iterator.observable || iterator;
    const dispatcher = iterator.dispatcher || ((req) => iterator.dispatch(req));
    const pipedObservable = operator !== undefined ? pipeObs(observable, operator) : observable;
    const liftedDispatcher = dispatchOperator !== undefined ? dispatchOperator(dispatcher) : dispatcher;
    return new LiftedIteratorImpl(liftedDispatcher, pipedObservable);
};
export const lift = (operator) => liftImpl(operator, undefined);
export const liftReq = (operator) => liftImpl(undefined, operator);
export function pipe(src, ...operators) {
    return operators.reduce((acc, next) => next(acc), src);
}
export const concatAll = (maxBufferSize = Number.MAX_SAFE_INTEGER) => lift(concatAllObs(maxBufferSize));
export const distinctUntilChanged = (equals) => lift(distinctUntilChangedObs(equals));
export const endWith = (value, ...values) => lift(endWithObs(value, ...values));
export const exhaust = () => lift(exhaustObs());
export const ignoreElements = () => lift(ignoreElementsObs());
export const keep = (predicate) => lift(keepObs(predicate));
export const map = (mapper) => lift(mapObs(mapper));
export const mergeAll = (options) => lift(mergeAllObs(options));
export const observe = (observer) => lift(observeObs(observer));
export const onComplete = (onComplete) => lift(onCompleteObs(onComplete));
export const onError = (onError) => lift(onErrorObs(onError));
export const onNext = (onNext) => lift(onNextObs(onNext));
export const repeat = (predicate) => lift(repeatObs(predicate));
export const retry = (predicate) => lift(retryObs(predicate));
export const scan = (scanner, initialValue) => lift(scanObs(scanner, initialValue));
export const share = (scheduler, replayCount) => lift(shareObs(scheduler, replayCount));
export const startWith = (value, ...values) => lift(startWithObs(value, ...values));
export const subscribeOn = (scheduler) => lift(subscribeOnObs(scheduler));
export const switchAll = () => lift(switchAllObs());
export const take = (count) => lift(takeObs(count));
export const takeLast = (count) => lift(takeLastObs(count));
export const takeWhile = (predicate) => lift(takeWhileObs(predicate));
export const throttle = (durationSelector) => lift(throttleObs(durationSelector));
export const throttleTime = (duration) => lift(throttleTimeObs(duration));
export const throttleFirst = (durationSelector) => lift(throttleFirstObs(durationSelector));
export const throttleFirstTime = (duration) => lift(throttleFirstTimeObs(duration));
export const throttleLast = (durationSelector) => lift(throttleLastObs(durationSelector));
export const throttleLastTime = (duration) => lift(throttleLastTimeObs(duration));
export const timeout = (duration) => lift(timeoutObs(duration));
export const withLatestFrom = (other, selector) => lift(withLatestFromObs(other, selector));
//# sourceMappingURL=index.js.map